#!/usr/bin/env python
#
# Data Quality Checker also known as DQC
#
# Read the README.md for more detailed information
#
#       Barcelona Supercomputing Center - Centro Nacional de Supercomputacion
#       Earth Sciences Department - Computational Earth Sciences
#     
#     Authors: 
#       Joan Sala Calero (joan.sala@bsc.es)

import sys
import traceback

# Local imports
from dqc_wrapper.utils.utils_files import *
from dqc_wrapper.utils.utils_pdf import *

"""
Class: Versioning and messages of the wrapper
"""
class checker_body:

    def __init__(self, name):
        self.name = name
        self.version = "0.0.3"
        self.help_mssg()

    def __version__(self):
        
        string = """        
          _____     ____     _____
         |  __ \   / __ \   / ____|
         | |  | | | |  | | | |
         | |  | | | |  | | | |
         | |__| | | |__| | | |____
         |_____/   \___\_\  \_____|
        
          Data Quality Checker - version {} (C3512 project)
        """.format(self.version)
        
        print(string.replace('\t', ''))

    def help_mssg(self):
        string = "\n#"+ "#" * 80 + "\n#\n"
        string += "#"+ " " * 2+"Data Quality Checker Help message - How to run the DQC:\n#\n"
        string += "#"+ " " * 4+"$ python checker.py <config_file.ini>\n#\n"
        string += "#" + "#" * 80 + "\n"

        self.help = string

"""
Class: Main interface to the checks
---
- Reads config file as unique commandline operator
- Loops over the different checks/sections
"""
def main(args):
    dqc = checker_body("testing")
    dqc.__version__()

    # Help display
    try:
        if len(args) > 1:
            raise Exception('Too many arguments')
        if (args[0] in ["--help","-h","help","sos","commands"]):
            raise Exception('Help message')
    except:
        print(dqc.help)
        sys.exit()        

    # Read configuration and check mandatory fields in the general section
    c = CheckerConfig(args[0])
    try:
        c.read_config()
    except Exception as e:
        print('ERROR: {}'.format(e))
        sys.exit()
        
    # Skip if zip file exists
    zipfname = get_zip_fname(c.res_dir, args[0])
    if os.path.exists(zipfname):
        print('INFO: Skipping {}'.format(zipfname))
        cleanup_directory(c.file_list['tmp_dir_split'])
        sys.exit()    

    # For each section/test
    files2zip = []
    err = False

    for s in c.config.sections():
        
        string = "." + "." * 80 + "\n"
        string += "Data Quality Checker [check={}]\n".format(s)        
        string += "." + "." * 80 + "\n"
        print(string)        

        # Each section is a different test available  
        try:
            test_res = None

            ## TEST 00
            if s == 'file_format':
                from dqc_wrapper.test00_file_format import FileFormatCheck
                file_format_ck = FileFormatCheck(c.file_list['original'])
                test_res = file_format_ck.main()

            ## TEST 01
            elif s == 'standard_compliance':
                from dqc_wrapper.test01_standard_compliance import StandardComplianceTest
                standard_ck = StandardComplianceTest(c.file_list['original'], c.forms_dir, c.cds_dataset, c.cds_variable)
                test_res = standard_ck.main()

            ## TEST 02
            elif s == 'spatial_completeness':
                from dqc_wrapper.test02_spatial_completeness import SpatialCompletenessTest
                spatial_completeness_ck = SpatialCompletenessTest(c.file_list['split'], c.varname, c.datatype, c.mf, c.mv)
                test_res = spatial_completeness_ck.main()                               

            ## TEST 03
            elif s == 'temporal_completeness':
                from dqc_wrapper.test03_temporal_completeness import TemporalCompletenessTest
                temporal_completeness_ck = TemporalCompletenessTest(
                    c.file_list['original'], c.forms_dir,
                    c.cds_dataset, c.cds_variable, c.origin, c.system, c.is_s)
                test_res = temporal_completeness_ck.main()

            ## TEST 04
            elif s == 'spatial_consistency':
                from dqc_wrapper.test04_spatial_consistency import SpatialConsistencyTest
                spatial_consistency_ck = SpatialConsistencyTest(
                    c.file_list['original'], c.grid_typ, c.grid_int, c.grid_map, c.dual_res)
                test_res = spatial_consistency_ck.main()

            ## TEST 05
            elif s == 'temporal_consistency':
                from dqc_wrapper.test05_temporal_consistency import TemporalConsistencyTest
                temporal_consistency_ck = TemporalConsistencyTest(
                    c.file_list['original'], c.time_s, c.time_g)
                test_res = temporal_consistency_ck.main()          

            ## TEST 06
            elif s == 'valid_ranges':
                from dqc_wrapper.test06_valid_ranges import ValidRangesTest
                valid_ranges_ck = ValidRangesTest(
                    c.res_dir, c.config_fname, c.file_list['split'], c.varname, c.vmin, c.vmax, c.datatype,
                    c.forms_dir, c.cds_dataset, c.cds_variable)
                test_res = valid_ranges_ck.main()          
                files2zip.append(test_res['plot'])

            ## TEST 07 - Extra (NetCDF converter - CF compliance)
            elif s == 'netcdf_converter':
                from dqc_wrapper.test01_standard_compliance import StandardComplianceTest
                standard_ck = StandardComplianceTest(c.file_list['nc_converter'], c.forms_dir, c.cds_dataset, c.cds_variable)
                test_res = standard_ck.main()

            else:
                raise Exception('Test {} is not available in the current checker implementation. Please check your config syntax')
        
        # Exceptions/Errors
        except KeyboardInterrupt:
            print_progress(100, 100)
            err=True
            print('Cancelled, exiting...')
            exit()
        except:            
            print_progress(100, 100)
            err=True
            print('ERROR: Processing section {}'.format(s))
            print('ERROR: {}'.format(traceback.format_exc()))

        # Append result + cleanup directory
        finally:
            c.save_config(s, test_res)

    # Cleanup directory where temp files were generated
    cleanup_directory(c.file_list['tmp_dir_split'])

    # Save config and PDF export
    try:
        # Save TXT result object
        res_file = save_result(c.config, c.res_dir, c.config_fname)

        # Save PDF report (result + log)
        if os.path.exists(res_file) and os.path.exists(c.log_file) and not (err):
            for s in c.config.sections():
                if not ('result' in s):
                    files2zip.append(save_pdf_report(s, res_file, c.log_file))
    except:
        print('ERROR: Could not save results. Check res_dir and log_dir permissions')

    # Zip all results
    if not(err):         
        zip_results(zipfname, files2zip)


if __name__ == "__main__":
    args = sys.argv[1:]
    sys.exit(main(args))
